@@ -18,7 +18,7 @@ _db = 'db'
 _obj = '_obj'
 _writer = 'writer'
 
-__all__ = ['root_path', 'pd_to_csv', 'pd_read_csv', 'parquet_encode', 'parquet_write', 'csv_encode', 'csv_write', 'pickle_dump', 'pickle_load', 'dictable_decoded']
+__all__ = ['root_path', 'pd_to_csv', 'pd_read_csv', 'parquet_encode', 'parquet_write', 'csv_encode', 'csv_write', 'pickle_dump', 'pickle_load', 'dictable_decode']
 
 
 # def encode(value):
@@ -124,17 +124,26 @@ def pd_read_csv(path):
         res = res.set_index('index')
     return res
 
-def dictable_decoded(path):
-    res = dictable(pd.read_parquet(path)).rename(lambda _col: _col[1:-1] if _col.startswith('"') else _col)
+def dictable_decode(df):
+    """
+    converts a dataframe with objects encoded into a dictable with decoded objects
+    :Parameters:
+    ------------
+    df: str/dataframe/dictable
+        items that can be converted into a dictable
+    """
+    res = dictable(df).rename(lambda _col: _col[1:-1] if _col.startswith('"') else _col)
     res = res.do(decode)
     return res
 
+
 _pd_read_csv = encode(pd_read_csv)
 _pd_read_parquet = encode(pd_read_parquet)
 _pd_read_npy = encode(pd_read_npy)
 _pickle_load = encode(pickle_load)
 _np_load = encode(np.load)
-_dictable_decoded = encode(dictable_decoded)
+_dictable_decode = encode(dictable_decode)
+
 
 def pickle_encode(value, path):
     """
@@ -146,7 +155,7 @@ def pickle_encode(value, path):
         path = path[:-1]
     if is_pd(value):
         path = root_path_check(path)
-        return dict(_obj = _pickle_load, path = pickle_dump(value, path + _pickle))
+        return dict(_obj = _pickle_load, path = pickle_dump(value, path if path.endswith(_pickle) else path + _pickle))
     elif is_arr(value):
         path = root_path_check(path)
         mkdir(path + _npy)
@@ -155,9 +164,8 @@ def pickle_encode(value, path):
     elif is_dict(value):
         res = type(value)(**{k : pickle_encode(v, '%s/%s'%(path,k)) for k, v in value.items()})
         if isinstance(value, dictable):
-            df = pd.DataFrame(res)
-            return dict(_obj = _dictable_decoded, 
-                        path =  pd_to_parquet_twice(df, path + _dictable))
+            return dict(_obj = _dictable_decode,
+                        df = dict(_obj = _pickle_load, path = pickle_dump(res, path if path.endswith(_dictable) else path + _dictable)))
         return res
     elif isinstance(value, (list, tuple)):
         return type(value)([pickle_encode(v, '%s/%i'%(path,i)) for i, v in enumerate(value)])
@@ -199,8 +207,8 @@ def parquet_encode(value, path, compression = 'GZIP'):
         res = type(value)(**{k : parquet_encode(v, '%s/%s'%(path,k), compression) for k, v in value.items()})
         if isinstance(value, dictable):
             df = pd.DataFrame(res)
-            return dict(_obj = _dictable_decoded, 
-                        path =  pd_to_parquet_twice(df, path + _dictable))
+            return dict(_obj = _dictable_decode,
+                        df = dict(_obj = _pd_read_parquet, path = pd_to_parquet_twice(df, path + _dictable)))
         return res
     elif isinstance(value, (list, tuple)):
         return type(value)([parquet_encode(v, '%s/%i'%(path,i), compression) for i, v in enumerate(value)])
@@ -232,8 +240,8 @@ def npy_encode(value, path, append = False):
         res = type(value)(**{k : npy_encode(v, '%s/%s'%(path,k), append = append) for k, v in value.items()})
         if isinstance(value, dictable):
             df = pd.DataFrame(res)
-            return dict(_obj = _dictable_decoded, 
-                        path =  pd_to_parquet_twice(df, path + _dictable))
+            return dict(_obj = _dictable_decode,
+                        df = dict(_obj = _pd_read_parquet, path = pd_to_parquet_twice(df, path + _dictable)))
         return res
     elif isinstance(value, (list, tuple)):
         return type(value)([npy_encode(v, '%s/%i'%(path,i), append = append) for i, v in enumerate(value)])
@@ -265,8 +273,8 @@ def csv_encode(value, path):
         res = type(value)(**{k : csv_encode(v, '%s/%s'%(path,k)) for k, v in value.items()})
         if isinstance(value, dictable):
             df = pd.DataFrame(res)
-            return dict(_obj = _dictable_decoded, 
-                        path =  pd_to_parquet_twice(df, path + _dictable))
+            return dict(_obj = _dictable_decode, 
+                        df = dict(_obj = _pd_read_csv, path = pd_to_csv(df, path)))
         return res
     elif isinstance(value, (list, tuple)):
         return type(value)([csv_encode(v, '%s/%i'%(path,i)) for i, v in enumerate(value)])
